/* ================================= Lisex ================================*/

/*! 
    \file 
    \brief Definicin de las funciones relativas a las listas de control de acceso.
*/

/*===============================Lisex=======================================*/

#define LISEXACLDEBUG(x) 

#include<linux/errno.h>
#include<linux/fs_acl.h>
#include <linux/stat.h>


/*! 
	\addtogroup vfs_acl
	
*/

/*@{*/

/*!
	\name Funciones privadas de vfs_acl
*/
/*@{*/

//! Determina si la entrada \a e de la ACL esta o no en uso.
#define ACL_INUSE(e) (e.mode != ACL_NULL_BITS)


/*! 
	\brief Busca una entrada libre dentro de la ACL.

    \param vfs_acl ACL donde se realizar la bsqueda.

    \return Posicin dentro de \a %vfs_acl donde se encontr el primer lugar libre, en caso de que la ACL est llena, este valor sera #MAX_ACL_LEN.
*/

int search_free_slot(struct vfs_acl * vfs_acl){
	int i;
	for(i=0;i<MAX_ACL_LEN;i++)
		if (!ACL_INUSE(vfs_acl->acl_entry[i])) 
			break;
	return i;
};


/*! 
	\brief Busca la entrada en la ACL que corresponde a al usuario o grupo id.

    \param vfs_acl ACL donde se realizar la bsqueda.
    \param id Identificacin del usuario o grupo a buscar.
    \param type Indica si \a id es de un usuario o de un grupo.

    \return Posicin dentro de \a %vfs_acl donde se encontr el usuario o grupo. Si esta es igual a #MAX_ACL_LEN significar que no se encontr tal usuario/grupo.
*/

int search_id(struct vfs_acl * vfs_acl, int id, int type)
{
	int i;
	for(i=0;i<MAX_ACL_LEN;i++)
		if (ACL_INUSE(vfs_acl->acl_entry[i]) && (ACL_TYPE(vfs_acl->acl_entry[i])==type) && (id == ACL_ID(vfs_acl->acl_entry[i])))
			break;
	return i;
};
/*@}*/


/*!
	\name Constantes (pblicas) de vfs_acl
	
	Debe considerarse que el nombre de la constante es pblico y su valor privado.
*/
/*@{*/

//ACL_OK se documenta tambin en fs_acl.h (mantener consistente)
/*!
	\def ACL_OK
	
	\brief Retorno de funciones que manejan las ACL.
	
	Indica que la operacin se realiz con xito. 
*/
/*@}*/


/*!
	\name Funciones de la interfaz de vfs_acl
*/
/*@{*/

/*! 
	\brief Calcula el modo tradicional de un inodo a partir de su ACL.

    \param vfs_acl ACL sobre la cual se calcular el modo tradicional.
    \param uid Identificacin del usuario dueo tradicional del inodo.
    \param gid Identificacin del grupo dueo tradicional del inodo.

    \return Un modo que se calcula a partir los permisos que tienen:
	     - el usuario dueno tradicional, 
	     - el grupo tradicional y AllGrup; 

     dentro de \a %vfs_acl.
*/
umode_t acl_calc_xmode(struct vfs_acl * vfs_acl, uid_t uid, gid_t gid){
	umode_t amode;
	umode_t mode;
	amode=acl_getmode(vfs_acl, uid, ACL_USR);
	mode = ((amode&ACL_READ_BIT)		?S_IRUSR:0)
		|((amode&ACL_WRITE_BIT)		?S_IWUSR:0)
		|((amode&ACL_EXEC_BIT)		?S_IXUSR:0);
	amode=acl_getmode(vfs_acl, gid, ACL_GRP);
	mode |= ((amode&ACL_READ_BIT)		?S_IRGRP:0)
		|((amode&ACL_WRITE_BIT)		?S_IWGRP:0)
		|((amode&ACL_EXEC_BIT)		?S_IXGRP:0);
	amode=acl_getmode(vfs_acl, ALLGRP, ACL_GRP);
	mode |= ((amode&ACL_READ_BIT)		?S_IROTH:0)
		|((amode&ACL_WRITE_BIT)		?S_IWOTH:0)
	        |((amode&ACL_EXEC_BIT)		?S_IXOTH:0);

    return mode;
}


/*! 
	\brief Calcula los permisos compatibles con el modo.

	Calcula los permisos compatibles con el modo utilizado en una ACL, a partir de un modo tradicional y de un entero que indica si son permisos de dueo o no. 

    \param xmode Modo tradicional.
    \param owner Los permisos que se calculan corresponden a un dueo o no.

    \return El modo en formato ACL.
*/
int xmode_to_acl(int xmode, int owner)
{
return   (((xmode&S_IROTH)?ACL_READ_BIT:0)
	|((xmode&S_IWOTH)?ACL_WRITE_BIT:0)
	|((xmode&S_IXOTH)?ACL_EXEC_BIT:0)
	| (owner?ACL_OWNER_BIT:0));
}


/*! 
	\brief Inicializa una ACL.

    \param vfs_acl Es la ACL a inicializar, se establece un valor nulo para los modos de las entradas de \a %vfs_acl y se asigna 0 en size::vfs_acl.
*/

void acl_aclinit(struct vfs_acl * vfs_acl)
{
	int i;
	for(i=0;i<MAX_ACL_LEN;i++)
	    vfs_acl->acl_entry[i].mode = ACL_NULL_BITS;
	vfs_acl->size=0;
};


//! Aun no est implementada, pero debera liberar la ACL que recibe como parmetro.

void acl_acldestroy(struct vfs_acl * vfs_acl)
{
}


/*! 
	\brief Establece el modo de un usuario o grupo en la ACL.

    En caso de que el usuario/grupo exista en la ACL, cambia su  modo siempre 
    que el modo pasado como parmetro sea un modo vlido. 
    Si no existe tal usuario/grupo y hay lugar en la ACL, lo agrega.

    \param vfs_acl ACL sobre la que se realizar la operacin.
    \param id Identificacin del usuario o grupo que se busca dentro de la ACL.
    \param mode Nuevo modo que deber quedar cargado en la ACL para el usuario/grupo con identificacion \a id luego de realizar la operacin, siempre que esto sea posible.
    \param type Indica si \a id representa la identificacin de un usuario o de un grupo.

    \return 
	    - #ACL_OK En caso de que el usuario o grupo ya exista en \a %vfs_acl y \a mode sea un modo vlido.
	    - #ACL_OK En caso de que el usuario o grupo no se encuentre en \a %vfs_acl pero que exista un lugar vaco y \a mode sea un modo vlido.
	    - #ACL_OK En caso de que el usuario o grupo no se encuentre en \a %vfs_acl pero que el modo sea el modo nulo.
	    - ENOMEM En caso de que el usuario o grupo no se encuentre en \a %vfs_acl y esta est llena.
*/

int acl_setmode(struct vfs_acl * vfs_acl, int id, int mode, int type){
int i;
	i = search_id(vfs_acl, id, type);
	if(i<MAX_ACL_LEN)
	{ 	/* el usr ya estaba, cambiamos su mode */
		if(mode != ACL_NULL_BITS)
		{
		    vfs_acl->acl_entry[i].mode = mode;
		    return ACL_OK;
		}
		else
		{
			vfs_acl->acl_entry[i].mode = ACL_NULL_BITS;
			(vfs_acl->size)--;
	        }
	}
	else	/*  es nuevo, buscamos un lugar vacio y se agrega a las acl*/
		if(mode)
		{
		    if((vfs_acl->size)<MAX_ACL_LEN)
		    {
			i=search_free_slot(vfs_acl);
			vfs_acl->acl_entry[i].mode = mode;
			vfs_acl->acl_entry[i].type = type;
			if(type==ACL_USR)
			    vfs_acl->acl_entry[i].id.uid = (uid_t) id;
			else
			    vfs_acl->acl_entry[i].id.gid = (gid_t) id;

			(vfs_acl->size)++;
			return ACL_OK;
		    }
		    else return -ENOMEM;
		} 
		else return ACL_OK; 
return 0;
};


/*! 
	\brief Obtiene el modo de un usuario/grupo.

    \param vfs_acl ACL sobre la cual se realizar la operacin.
    \param id Identifiacin del usuario del cual se desea obtener su modo.
    \param type Indica si \a id se refiere a la identificacin de un grupo o de un usuario.
    \return
	    - El modo del usuario/grupo con identificacin \a id en la ACL \a %vfs_acl.
*/

int acl_getmode(struct vfs_acl * vfs_acl, int id, int type){
    int i;
    if((i=search_id(vfs_acl,id,type)) < MAX_ACL_LEN)
	return vfs_acl->acl_entry[i].mode;
return ACL_NULL_BITS;
}


/*! 
	\brief Devuelve la cantidad de entradas vlidas que existen en la ACL.    
 
    \param vfs_acl ACL de la cual se obtiene la cantidad de entradas vlidas almacenadas.
    \return La cantidad de entradas vlidas dentro de la ACL.
*/
inline int acl_getsize(struct vfs_acl * vfs_acl){
    return vfs_acl->size;
};

/*@}*/


/*!
	\name Iterador para las acl_entry de un objeto vfs_acl
	
	Tambin forma parte de la interfaz de vfs_acl.
	
	Ver #acl_for_each para un ejemplo.
*/
/*@{*/


//ACL_IT_ERROR y acl_for_each se documentan nuevamente en fs_acl.h (mantener consistente)
/*!
	\def ACL_IT_ERROR
	
	\brief Retorno de las funciones que iteran sobre la ACL de un inodo.

    Indica que no existen ms entradas vlidas en la ACL a partir de la posicin pasada como parmetro.


*/

/*!
	\def acl_for_each(acl,i)

	\brief Macro que itera entre todas las entradas vlidas de una ACL. 

    \param acl ACL sobre la cual se va a iterar.
    \param i Es el puntero iterador.
*/

/*!
	\example iterador_acl_entry.c
	
   	Este ejemplo muestra cmo usar el iterador para recorrer las entradas de una ACL.
*/


/*! 
	\brief Busca la primer entrada vlida dentro de la ACL.

    \param vfs_acl ACL donde se realizar la bsqueda.

    \return 
	    - La posicin dentro de \a %vfs_acl donde se encontr la primer entrada vlida. 
	    - #ACL_IT_ERROR En caso de que \a %vfs_acl este vacia.
*/

acl_iterator acl_first(struct vfs_acl * vfs_acl){
int i;
LISEXACLDEBUG(if(!vfs_acl) printk("ACL::acl_first: Bad acl!\n"));
	for(i=0;i<MAX_ACL_LEN;i++)
		if (ACL_INUSE(vfs_acl->acl_entry[i])) 
			break;
	return ((i<MAX_ACL_LEN)?i:ACL_IT_ERROR);
};


/*! 
	\brief Busca la prxima entrada vlida en la ACL a partir de una posicin que se pasa como parmetro.

    \param vfs_acl  ACL donde se realizar la bsqueda.
    \param it Posicin a partir de la cual se realizar la bsqueda, previamente obtenido con acl_first() o acl_next().
    
    \return 
    	- La posicin dentro de \a %vfs_acl donde se encontr la prxima entrada vlida. 
	    - #ACL_IT_ERROR En caso de que \a it sea la posicin de la ltima entrada vlida dentro de \a %vfs_acl.
*/

acl_iterator acl_next(struct vfs_acl * vfs_acl, acl_iterator it){
LISEXACLDEBUG(if(!vfs_acl) printk("ACL::acl_next: Bad acl!\n"));
	for(it=it+1;it<MAX_ACL_LEN;it++)
		if (ACL_INUSE(vfs_acl->acl_entry[it]))
		    return it;
	return ACL_IT_ERROR;
};



/*!
	\brief Devuelve una entrada de la ACL.

    \param vfs_acl ACL a la que pertenece la entrada que se devuelve.
    \param it Posicin, dentro de \a %vfs_acl, de la entrada que se quiere obtener. 

    \return La entrada ubicada en la posicin \a it de \a %vfs_acl.
*/
struct acl_entry acl_getentry(struct vfs_acl * vfs_acl, acl_iterator it){
LISEXACLDEBUG(if(!vfs_acl) printk("ACL::acl_getentry: Bad acl!\n"));
LISEXACLDEBUG(if(it>=MAX_ACL_LEN) printk("ACL::acl_getentry: acl_iterator out of range!\n"));
return vfs_acl->acl_entry[it];
};

/*@}*/
/*@}*/

#ifdef __KERNEL__
#include <linux/fs.h>
#include <linux/mm.h>
#include <asm/uaccess.h>
#include <linux/sched.h>
#include <linux/smp_lock.h>
//#include <linux/dnotify.h>





/*!
	\ingroup newsyscall
	
	\brief Llamada al sistema acladd. Agrega permisos a un usuario o grupo en la ACL.
	
	Agrega permisos a un usuario o grupo en la ACL del inodo de un archivo, siempre que esto sea posible.
	
    \param filename Nombre del archivo sobre el cual se realizar la operacion.
    \param id Identificacin del usuario/grupo al que se le agregan permisos a la ACL del inodo del archivo filename.
    \param mode Permisos que se le agregarn a los permisos que tenga el usuario/grupo id en la ACL del inodo del archivo filename.
    \param type Determina si id es la identificacin de un usuario o de un grupo.
    \post
    - Se agrega a la ACL del archivo filename el usuario/grupo con identificacin id y los permisos mode.
    \return 
    - Un nmero distinto de cero si filename no es un nombre valido.
    - -EROFS Si el archivo filename es read-only.
    - -EPERM Si el archivo filename es inmutable o append.???
    - -EACCES Si el usuario que realiza la operacin no es dueo del archivo filename.
    - -EBUSY Si el archivo filename esta abierto en el sistema.
    - -ACL_OK Si la operacin se realiz con xito.
    - -ENOMEM Si la ACL del inodo del archivo filename estaba llena y el usuario/grupo id no tena ningn permiso en la ACL de este inodo antes de realizarse la operacin.
*/


asmlinkage long sys_acladd(char * filename, int id, int mode, int type)
{
	struct nameidata nd;
	int error;
	struct vfs_acl * acl;
	struct inode * inode;
	time_t now; 
	LISEXDEBUG(struct acl_entry acl_entry;	acl_iterator i;)

	LISEXDEBUG(printk("sys_acladd file: %s id: %d mode: %d tipo: %d\n", filename, id, mode, type);)
	error = -EINVAL;	
	now = CURRENT_TIME;	
	lock_kernel();
	error = user_path_walk(filename, &nd);
    	if (error) 
	    goto out;
	inode = nd.dentry->d_inode;
	acl   = &inode->vfs_acl;
	error = -EROFS;
	if (IS_RDONLY(inode))
		goto release_and_out;
	error = -EPERM;
	if (IS_IMMUTABLE(inode) || IS_APPEND(inode))
		goto release_and_out;
	error = -EACCES;
	
	
	if(permission(nd.dentry->d_inode, ACL_OWNER_BIT))
		goto release_and_out;
	error = -EBUSY;
	if(is_open(nd.dentry->d_inode))
		goto release_and_out;
	error = acl_setmode(acl, id, acl_getmode(acl,id,type) | mode, type);
	inode->i_ctime = now;
	mark_inode_dirty(inode);
	/* Lisex inode_dir_notify error??*/
	//inode_dir_notify(inode, DN_ATTRIB);

	/*----------BEG  SACAR LISEX--------*/
	/*acl_for_each(&inode->vfs_acl, i)
	{
	    acl_entry = acl_getentry(&inode->vfs_acl,i);
	    printk("\tAcl N: %d ",  i);
	    printk("\tAcl ID: %d ",  ACL_ID(acl_entry));
	    printk("\tAcl MODE:%d ", ACL_MODE(acl_entry));
	    printk("\tAcl TYPE:%d  ** ", ACL_TYPE(acl_entry));
	}*/
	/*----------END  SACAR LISEX--------*/



release_and_out:
	path_release(&nd);	
out:
	unlock_kernel();
	return error;
}


/*! 
	\ingroup newsyscall
	
    \brief Llamada al sistema acldel. Quita permisos a un usuario o un grupo de una ACL.
    
    Quita permisos a un usuario o un grupo en la ACL del inodo de un archivo, siempre que esto sea posible.
    
    \param filename Nombre del archivo sobre el cual se realizar la operacin.
    \param id Identificacin del usuario/grupo al que se le quitan permisos en la ACL del inodo del archivo filename.
    \param mode Permisos que se le quitarn a los permisos que tenga el usuario/grupo id en la ACL del inodo del archivo filename. 
    \param type Determina si id es la identificacin de un usuario o de un grupo.
    \post La ACL del archivo filename cambia por una nueva ACL donde el 
    usuario/grupo con identificacin id tiene los permisos que tena antes
    menos los permisos que estn en mode, el resto de la ACL queda sin cambios.
    Si id es el usuario dueno tradicional del archivo filename, y mode contiene el permiso de dueo, la ACL y el archivo cambian de la siguiente forma:
    El dueo tradicional de filename pasa a ser ROOT, se agrega a la ACL el usuario ROOT con permisos de dueo. Se quitan de la ACL los permisos del usuario id que estn en mode.
    \return 
    - Un nmero distinto de cero si filename no es un nombre valido.
    - -EROFS Si el inodo del archivo filename es read-only.
    - -EPERM Si el inodo del archivo filename es inmutable o append?????
    - -EPERM Si se intenta borrar al grupo ROOTGRP del conjunto de grupos dueos de la ACL del archivo filename.
    - -EACCES Si el usuario que realiza la operacin no es dueo de \a filename.    
    - -EBUSY Si el archivo filename esta abierto en el sistema.
    - -EINVAL Si el usuario/grupo id no exista en la ACL del inodo del archivo filename antes de realizarse la operacin.
    - -ENOMEM Si la operacin no pudo agregar a ROOT en la ACL en el caso de que id sea el dueo tradicional de filename. 
    - -ACL_OK Si la operacin se realiz con xito.
*/

asmlinkage long sys_acldel(char * filename, int id, int mode, int type)
{
	struct nameidata nd;
	int error;
	struct vfs_acl * acl;
	struct inode * inode;
	int muser;
	time_t now; 
	
	now = CURRENT_TIME;	
	lock_kernel();
	error = user_path_walk(filename, &nd);
    	if (error) 
	    goto out;
	inode = nd.dentry->d_inode;
	acl   = &inode->vfs_acl;
	error = -EROFS;
	if (IS_RDONLY(inode))
		goto release_and_out;
	error = -EPERM;
	if (IS_IMMUTABLE(inode) || IS_APPEND(inode))
		goto release_and_out;
	error = -EPERM;	
	if (type == ACL_GRP && (id==ROOTGRP) && (mode & ACL_OWNER_BIT)) 
		goto release_and_out;
	error = -EACCES;
	if(permission(nd.dentry->d_inode, ACL_OWNER_BIT))
		goto release_and_out;
	error = -EBUSY;
	if(is_open(nd.dentry->d_inode))
		goto release_and_out;
	if(type)
	    if( ((uid_t)id) == inode->i_uid)
		if(mode && ACL_OWNER_BIT){
		    muser = acl_getmode(acl,id,type);
		    acl_setmode(acl,id, muser & ~mode , ACL_USR);
		    if((error = acl_setmode(acl, ROOTID,acl_getmode(acl,ROOTID,type)|ACL_OWNER_BIT, ACL_USR))){
			acl_setmode(acl,id, muser, ACL_USR);
			goto release_and_out;
		    }
		    inode->i_uid = 0;
		    goto release_and_out;
		}
	error = acl_setmode(acl, id, acl_getmode(acl,id,type) & (~mode), type);
	inode->i_ctime = now;
	mark_inode_dirty(inode);
	/* Lisex inode_dir_notify error??*/
	//inode_dir_notify(inode, DN_ATTRIB);
release_and_out:
	path_release(&nd);	
out:
	unlock_kernel();
	return error;
}
#endif
