/*! \file stat.c
    \brief Llamadas al sistema de la familia stat 
    
    - Estas llamadas son las siguientes:
	-# sys_stat()
	-# sys_newstat() 
	-# sys_lstat()
	-# sys_newlstat() 
	-# sys_fstat()
	-# sys_newfstat() 
	-# sys_stat64() 
	-# sys_lstat64()
	-# sys_aclstat()
	-# sys_aclfstat()

	La llamada sys_newstat() es una implementacin ms actual de la llamada sys_stat(), esta ltima sigue existiendo por problmas de compatibilidad.
	Lo mismo ocurre con sys_newlstat(), sys_newfstat().
*/

/*
 *  linux/fs/stat.c
 *
 *  Copyright (C) 1991, 1992  Linus Torvalds
 */

#include <linux/config.h>
#include <linux/mm.h>
#include <linux/errno.h>
#include <linux/file.h>
#include <linux/smp_lock.h>
#include <linux/highuid.h>

#include <asm/uaccess.h>
#include<linux/fs_acl.h>

/*
 * Revalidate the inode. This is required for proper NFS attribute caching.
 */
static __inline__ int
do_revalidate(struct dentry *dentry)
{
	struct inode * inode = dentry->d_inode;
	if (inode->i_op && inode->i_op->revalidate)
		return inode->i_op->revalidate(dentry);
	return 0;
}


#if !defined(__alpha__) && !defined(__sparc__) && !defined(__ia64__) && !defined(CONFIG_ARCH_S390) && !defined(__hppa__) && !defined(__x86_64__)

/*
 * For backward compatibility?  Maybe this should be moved
 * into arch/i386 instead?
 */

//! Esta funcin es utilizada por las llamadas sys_stat() y sys_fstat().
/*!
	\ingroup auxfunc
	 
    Se cambi el cdigo que realizaba el clculo del modo, el nuevo cdigo calcula el modo utilizando la ACL del inodo que se pasa como prametro.
    Esta funcin carga en statbuf informacin que obtiene del inodo inode.
    
    \param inode Inodo sobre el cual se realiza la operacin.
    \param statbuf Buffer donde se cargan los resultados.
    \return
    - -EFAULT Si no se pudo concretar la copia de los datos a statbuf
    - 0 Si la operacin se realiz con xito.
*/
static int cp_old_stat(struct inode * inode, struct __old_kernel_stat * statbuf)
{
	static int warncount = 5;
	struct __old_kernel_stat tmp;

	if (warncount > 0) {
		warncount--;
		printk(KERN_WARNING "VFS: Warning: %s using old stat() call. Recompile your binary.\n",
			current->comm);
	} else if (warncount < 0) {
		/* it's laughable, but... */
		warncount = 0;
	}

	tmp.st_dev = kdev_t_to_nr(inode->i_dev);
	tmp.st_ino = inode->i_ino;
	
/*===============================Lisex=======================================*/
	tmp.st_mode = (inode->i_mode & ~S_IRWXUGO) |acl_calc_xmode(&inode->vfs_acl, inode->i_uid, inode->i_gid);
	
	tmp.st_nlink = inode->i_nlink;
	SET_OLDSTAT_UID(tmp, inode->i_uid);
	SET_OLDSTAT_GID(tmp, inode->i_gid);
	tmp.st_rdev = kdev_t_to_nr(inode->i_rdev);
#if BITS_PER_LONG == 32
	if (inode->i_size > MAX_NON_LFS)
		return -EOVERFLOW;
#endif	
	tmp.st_size = inode->i_size;
	tmp.st_atime = inode->i_atime;
	tmp.st_mtime = inode->i_mtime;
	tmp.st_ctime = inode->i_ctime;
	return copy_to_user(statbuf,&tmp,sizeof(tmp)) ? -EFAULT : 0;
}

#endif
//! Esta funcin es utilizada por las llamadas sys_newstat() y sys_newfstat().
/*! 
	\ingroup auxfunc
	
    Se cambi el cdigo que realizaba el clculo del modo, el nuevo cdigo calcula el modo utilizando la ACL del inodo que se pasa como prametro.
    Los prametros y lo que devuelve es igual a cp_old_stat().
*/

static int cp_new_stat(struct inode * inode, struct stat * statbuf)
{
	struct stat tmp;
	unsigned int blocks, indirect;

	memset(&tmp, 0, sizeof(tmp));
	tmp.st_dev = kdev_t_to_nr(inode->i_dev);
	tmp.st_ino = inode->i_ino;

/*===============================Lisex=======================================*/
	tmp.st_mode = (inode->i_mode & ~S_IRWXUGO) |acl_calc_xmode(&inode->vfs_acl, inode->i_uid, inode->i_gid);

	tmp.st_nlink = inode->i_nlink;
	SET_STAT_UID(tmp, inode->i_uid);
	SET_STAT_GID(tmp, inode->i_gid);
	tmp.st_rdev = kdev_t_to_nr(inode->i_rdev);
#if BITS_PER_LONG == 32
	if (inode->i_size > MAX_NON_LFS)
		return -EOVERFLOW;
#endif	
	tmp.st_size = inode->i_size;
	tmp.st_atime = inode->i_atime;
	tmp.st_mtime = inode->i_mtime;
	tmp.st_ctime = inode->i_ctime;
/*
 * st_blocks and st_blksize are approximated with a simple algorithm if
 * they aren't supported directly by the filesystem. The minix and msdos
 * filesystems don't keep track of blocks, so they would either have to
 * be counted explicitly (by delving into the file itself), or by using
 * this simple algorithm to get a reasonable (although not 100% accurate)
 * value.
 */

/*
 * Use minix fs values for the number of direct and indirect blocks.  The
 * count is now exact for the minix fs except that it counts zero blocks.
 * Everything is in units of BLOCK_SIZE until the assignment to
 * tmp.st_blksize.
 */
#define D_B   7
#define I_B   (BLOCK_SIZE / sizeof(unsigned short))

	if (!inode->i_blksize) {
		blocks = (tmp.st_size + BLOCK_SIZE - 1) / BLOCK_SIZE;
		if (blocks > D_B) {
			indirect = (blocks - D_B + I_B - 1) / I_B;
			blocks += indirect;
			if (indirect > 1) {
				indirect = (indirect - 1 + I_B - 1) / I_B;
				blocks += indirect;
				if (indirect > 1)
					blocks++;
			}
		}
		tmp.st_blocks = (BLOCK_SIZE / 512) * blocks;
		tmp.st_blksize = BLOCK_SIZE;
	} else {
		tmp.st_blocks = inode->i_blocks;
		tmp.st_blksize = inode->i_blksize;
	}
	return copy_to_user(statbuf,&tmp,sizeof(tmp)) ? -EFAULT : 0;
}


#if !defined(__alpha__) && !defined(__sparc__) && !defined(__ia64__) && !defined(CONFIG_ARCH_S390) && !defined(__hppa__) && !defined(__x86_64__)
/*
 * For backward compatibility?  Maybe this should be moved
 * into arch/i386 instead?
 */


/*! 
	\ingroup syscall

	\brief Llamada al sistema stat.
	
    Devuelve informacin de estado de un archivo.	
    Esta llamada utiliza la funcin cp_old_stat() la cual fue modificada para cumplir con la nueva especificacin.

    \param filename Archivo del cual se piede la informacin.
    \param statbuf Apunta a una estructura donde se devuelven los resultados de la llamada.
    
    \post     
    En el prametro \a statbuf queda cargada informacin sore \a filename, dentro de la cual se encuentran los permisos del usuario dueo tradicional, el grupo primario tradicional y los permisos del grupo #ALLGRP.
     
    \return 
    - Un nmero distinto de cero si \a filename no es un nombre vlido.???
    - -EACCES Si el usuario que realiza la operacin no tiene permiso de lectura sobre \a filename.
    - Lo que devuelva cp_old_stat() en cualquier otro caso.
*/
asmlinkage long  sys_stat(char * filename, struct __old_kernel_stat * statbuf)
{
	struct nameidata nd;
	int error;

	error = user_path_walk(filename, &nd);
	
	if(error)
	    goto out;
/*===============================Lisex=======================================*/	
	
	error = permission(nd.dentry->d_inode, ACL_READ_BIT);
	if(error)
		goto path_relase_and_out;
	
	/* Para NFS */
	error = do_revalidate(nd.dentry);
	
	if (error)
	    goto path_relase_and_out;

	error = cp_old_stat(nd.dentry->d_inode, statbuf);


path_relase_and_out: path_release(&nd);

out: return error;
}

#endif


/*!
	\addtogroup syscall
	@{
*/
//! Llamada al sistema newstat. dem sys_stat() con la diferencia que esta llamada utiliza la funcin cp_new_stat().

asmlinkage long  sys_newstat(char * filename, struct stat * statbuf)
{
	struct nameidata nd;
	int error;

	error = user_path_walk(filename, &nd);

	if(error)
	    goto out;
/*===============================Lisex=======================================*/
	
	error = permission(nd.dentry->d_inode, ACL_READ_BIT);
	if(error)
		goto path_relase_and_out;
	
	
	error = do_revalidate(nd.dentry);
	
	if (error)
	    goto path_relase_and_out;

	error = cp_new_stat(nd.dentry->d_inode, statbuf);


path_relase_and_out: path_release(&nd);

out: return error;

}

#if !defined(__alpha__) && !defined(__sparc__) && !defined(__ia64__) && !defined(CONFIG_ARCH_S390) && !defined(__hppa__) && !defined(__x86_64__)

/*
 * For backward compatibility?  Maybe this should be moved
 * into arch/i386 instead?
 */


/*! 
	\ingroup syscall

	\brief Llamada al sistema lstat.	
	
    Devuelve informacin del enlace que se obtiene al seguir los enlaces del path que se pasa como parmetro.
    Esta llamada utiliza la funcin cp_old_stat() la cual fue modificada para cumplir con la nueva especificacin.

    \param filename Enlace del cual se piede la informacin.
    \param statbuf Apunta a una estructura donde se devuelven los resultados de lstat.

    \post dem sys_stat().

    \return  dem sys_stat().

*/
asmlinkage long sys_lstat(char * filename, struct __old_kernel_stat * statbuf)
{
	struct nameidata nd;
	int error;

	error = user_path_walk_link(filename, &nd);

	if(error)
	    goto out;
/*===============================Lisex=======================================*/	
	
	error = permission(nd.dentry->d_inode, ACL_READ_BIT);
	if(error)
		goto path_relase_and_out;
	
	error = do_revalidate(nd.dentry);
	
	if (error)
	    goto path_relase_and_out;

	error = cp_old_stat(nd.dentry->d_inode, statbuf);


path_relase_and_out: path_release(&nd);

out: return error;
}

#endif


//! Llamada al sistema newlstat. dem sys_lstat() con la diferencia que esta llamada utiliza la funcin cp_new_stat().

asmlinkage long sys_newlstat(char * filename, struct stat * statbuf)
{
	struct nameidata nd;
	int error;

	error = user_path_walk_link(filename, &nd);
	if(error)
	    goto out;
/*===============================Lisex=======================================*/	
	
	error = permission(nd.dentry->d_inode, ACL_READ_BIT);
	if(error)
		goto path_relase_and_out;

	
	error = do_revalidate(nd.dentry);
	
	if (error)
	    goto path_relase_and_out;

	error = cp_new_stat(nd.dentry->d_inode, statbuf);


path_relase_and_out: path_release(&nd);

out: return error;
}

#if !defined(__alpha__) && !defined(__sparc__) && !defined(__ia64__) && !defined(CONFIG_ARCH_S390) && !defined(__hppa__) && !defined(__x86_64__)

/*
 * For backward compatibility?  Maybe this should be moved
 * into arch/i386 instead?
 */


/*! 
	\ingroup syscall

	\brief Llamada al sistema fstat.

    Devuelve informacin del inodo  al que apunta el descripor de archivo que se pasa como parmetro.
	
    Esta llamada utiliza la funcin cp_old_stat() la cual fue modificada para cumplir con la nueva especificacin.

    \param fd Descriptor de archivo que apunta al inodo del cual se piede informacin.
    \param statbuf Apunta a una estructura donde se devuelven los resultados de la llamada.

    \post En el parmetro \a statbuf queda cargada informacin sore el archivo al que apunta \a fd, dentro de la cual se encuentran los permisos del usuario dueno tradicional, el grupo primario tradicional y los permisos del grupo #ALLGRP. 

    \return 
    - -EBADF Si \a fd no apunta a un inodo vlido. ???
    - -EACCES Si el usuario que realiza la operacin no tiene permiso de lectura sobre el archivo al que apunta \a fd.
    - Lo que devuelva cp_old_stat() en cualquier otro caso.
*/

asmlinkage long sys_fstat(unsigned int fd, struct __old_kernel_stat * statbuf)
{
	struct file * f;
	int error = -EBADF;
	struct dentry * dentry;

	f = fget(fd);
	if (!f) 
	    goto out;
	
	dentry = f->f_dentry;
/*===============================Lisex=======================================*/
	error = permission(dentry->d_inode, ACL_READ_BIT);
	if(error)
		goto fputf_and_out;
	
	error = do_revalidate(dentry);
	
	if (error)
	    goto fputf_and_out;
	
	error = cp_old_stat(dentry->d_inode, statbuf);
	
fputf_and_out: 
	fput(f);	
out:
	return error;
}

#endif

//! Llamada al sistema newfstat. dem sys_fstat() con la diferencia que esta llamada utiliza la funcin cp_new_stat().

asmlinkage long sys_newfstat(unsigned int fd, struct stat * statbuf)
{
	struct file * f;
	struct dentry * dentry;
	int error = -EBADF;

	f = fget(fd);
	if (!f) 
	    goto out;
	    
	dentry = f->f_dentry;
/*===============================Lisex=======================================*/
	error = permission(dentry->d_inode, ACL_READ_BIT);
	if(error)
		goto fputf_and_out;
	
	error = do_revalidate(dentry);
	
	if (error)
	    goto fputf_and_out;
	
	error = cp_new_stat(dentry->d_inode, statbuf);
	
fputf_and_out: 
	fput(f);	
out:	
	return error;
}
/*@}*/


asmlinkage long sys_readlink(const char * path, char * buf, int bufsiz)
{
	struct nameidata nd;
	int error;

	if (bufsiz <= 0)
		return -EINVAL;

	error = user_path_walk_link(path, &nd);
	if (!error) {
		struct inode * inode = nd.dentry->d_inode;

		error = -EINVAL;
		if (inode->i_op && inode->i_op->readlink &&
		    !(error = do_revalidate(nd.dentry))) {
			UPDATE_ATIME(inode);
			error = inode->i_op->readlink(nd.dentry, buf, bufsiz);
		}
		path_release(&nd);
	}
	return error;
}


/* ---------- LFS-64 ----------- */
#if !defined(__alpha__) && !defined(__ia64__) && !defined(__mips64) && !defined(__x86_64__) && !defined(CONFIG_ARCH_S390X)

/*! 
	\ingroup auxfunc
	
	\brief Esta funcin es utilizada por las llamadas sys_stat64(), sys_fstat64() y sys_lstat64().

    Se cambi cdigo que realizaba el clculo del modo, el nuevo cdigo calcula el modo utilizando la ACL del inodo que se pasa como parmetro.
    Los parmetros y lo que devuelve es igual a cp_old_stat().
*/

static long cp_new_stat64(struct inode * inode, struct stat64 * statbuf)
{
	struct stat64 tmp;
	unsigned int blocks, indirect;

	memset(&tmp, 0, sizeof(tmp));
	tmp.st_dev = kdev_t_to_nr(inode->i_dev);
	tmp.st_ino = inode->i_ino;
#ifdef STAT64_HAS_BROKEN_ST_INO
	tmp.__st_ino = inode->i_ino;
#endif

/*===============================Lisex=======================================*/
	tmp.st_mode = (inode->i_mode & ~S_IRWXUGO) |acl_calc_xmode(&inode->vfs_acl, inode->i_uid, inode->i_gid);

	tmp.st_nlink = inode->i_nlink;
	tmp.st_uid = inode->i_uid;
	tmp.st_gid = inode->i_gid;
	tmp.st_rdev = kdev_t_to_nr(inode->i_rdev);
	tmp.st_atime = inode->i_atime;
	tmp.st_mtime = inode->i_mtime;
	tmp.st_ctime = inode->i_ctime;
	tmp.st_size = inode->i_size;
/*
 * st_blocks and st_blksize are approximated with a simple algorithm if
 * they aren't supported directly by the filesystem. The minix and msdos
 * filesystems don't keep track of blocks, so they would either have to
 * be counted explicitly (by delving into the file itself), or by using
 * this simple algorithm to get a reasonable (although not 100% accurate)
 * value.
 */

/*
 * Use minix fs values for the number of direct and indirect blocks.  The
 * count is now exact for the minix fs except that it counts zero blocks.
 * Everything is in units of BLOCK_SIZE until the assignment to
 * tmp.st_blksize.
 */
#define D_B   7
#define I_B   (BLOCK_SIZE / sizeof(unsigned short))

	if (!inode->i_blksize) {
		blocks = (tmp.st_size + BLOCK_SIZE - 1) >> BLOCK_SIZE_BITS;
		if (blocks > D_B) {
			indirect = (blocks - D_B + I_B - 1) / I_B;
			blocks += indirect;
			if (indirect > 1) {
				indirect = (indirect - 1 + I_B - 1) / I_B;
				blocks += indirect;
				if (indirect > 1)
					blocks++;
			}
		}
		tmp.st_blocks = (BLOCK_SIZE / 512) * blocks;
		tmp.st_blksize = BLOCK_SIZE;
	} else {
		tmp.st_blocks = inode->i_blocks;
		tmp.st_blksize = inode->i_blksize;
	}
	return copy_to_user(statbuf,&tmp,sizeof(tmp)) ? -EFAULT : 0;
}


/*!
	\addtogroup syscall
	@{
*/
//! Llamada al sistema stat64. dem sys_stat() pero para los sistemas de archivos de 64 bits.

asmlinkage long sys_stat64(char * filename, struct stat64 * statbuf, long flags)
{
	struct nameidata nd;
	int error;

	error = user_path_walk(filename, &nd);

	if(error)
	    goto out;
/*===============================Lisex=======================================*/	

	
	error = permission(nd.dentry->d_inode, ACL_READ_BIT);
	if(error)
		goto path_relase_and_out;

	error = do_revalidate(nd.dentry);
	
	if (error)
	    goto path_relase_and_out;

	error = cp_new_stat64(nd.dentry->d_inode, statbuf);


path_relase_and_out: path_release(&nd);

out: return error;

}


//! Llamada al sistema lstat64. dem sys_lstat() pero para los fileystems de 64 bits.

asmlinkage long sys_lstat64(char * filename, struct stat64 * statbuf, long flags)
{
	struct nameidata nd;
	int error;

	error = user_path_walk_link(filename, &nd);

	if(error)
	    goto out;

/*===============================Lisex=======================================*/	

	
	error = permission(nd.dentry->d_inode, ACL_READ_BIT);
	if(error)
		goto path_relase_and_out;


	error = do_revalidate(nd.dentry);
	
	if (error)
	    goto path_relase_and_out;

	error = cp_new_stat64(nd.dentry->d_inode, statbuf);


path_relase_and_out: path_release(&nd);

out: return error;

}


//! Llamada al sistema fstat64. dem sys_fstat() pero para los sistemas de archivos de 64 bits.

asmlinkage long sys_fstat64(unsigned long fd, struct stat64 * statbuf, long flags)
{
	struct file * f;
	int error = -EBADF;
	struct dentry * dentry;
	
	f = fget(fd);
	if (!f) 
	    goto out;
	
	dentry = f->f_dentry;

/*===============================Lisex=======================================*/	
	
	error = permission(dentry->d_inode, ACL_READ_BIT);
	if(error)
		goto fputf_and_out;
	
	error = do_revalidate(dentry);
	
	if (error)
	    goto fputf_and_out;
	
	error = cp_new_stat64(dentry->d_inode, statbuf);
	
fputf_and_out:
	 fput(f);	
out:	
	return error;
}

#endif /* LFS-64 */
/*@}*/


/*===============================Lisex=======================================*/

/*! 
	\brief Carga en una estructura los datos de la ACL de un inodo.

	\ingroup newauxfunc
	
    Carga en una estructura los datos de la ACL de un inodo que se pasa como parmetro.
    
    Esta funcin es utilizada por las llamadas sys_aclstat() y sys_aclfstat().

    \param inode Inodo del cual se quiere obtener la ACL.
    \param acl_statbuf Estructura donde se cargan los resultados
    \param len Cantidad de entradas de la ACL que se piden obtener como resultado.
    
    \result
    	- -ENVAL Si len es < 0 o si len es menor que la cantidad de entradas vlidas que se encuentran en la ACL de inode.
	    - n Donde n es la cantidad de entradas vlidas que hay en la ACL del inodo inode, si len es 0. 
		- -EFAULF Si no se pudo realizar la carga de la ACL en statbuf.
		-  m Si la operacin se realiz con xito. m es la cantidad de entradas validas que se cargaron en statbuf. 
*/

static int cp_aclstat(struct inode * inode, struct acl_entry * acl_statbuf, int len) 
{
    struct acl_entry tmp[MAX_ACL_LEN];
    acl_iterator i;
    int j=0;

    if(len < 0 || len < acl_getsize(&inode->vfs_acl)) /*si es negativo o si no alcanza*/
	return -EINVAL;
    if(len)		/*si no es cero*/
    {
	acl_for_each(&inode->vfs_acl,i){
	    if(j>len) break;
	    tmp[j++]=acl_getentry(&inode->vfs_acl,i);	/*\bug deberiamos borrar los bits que no le interesan al usr!*/
	}
        if(copy_to_user(acl_statbuf,tmp,j * sizeof(struct acl_entry))) 
	    return -EFAULT;
    }
    return acl_getsize(&inode->vfs_acl);
}

/*===============================Lisex=======================================*/
//! Llamada al sistema aclstat. Devuelve informacin almacenada en una ACL.
/*!
	\ingroup newsyscall
	
    Esta llamada utiliza la funcin cp_aclstat().
    
    Devuelve informacin almacenada en la ACL de un archivo, siempre que el usuario que pide dicha informacin tiene permisos de lectura sobre el archivo.
    
    \param filename Archivo del cual se piede la informacin.
    \param acl_statbuf Apunta a una estructura donde se devuelven los resultados de aclstat.
    \param len Cantidad de entradas de la ACL que se piden obtener como resultado.
    \post
    - En el parmetro statbuf quedan cargadas todas las entradas vlidas de la ACL del archivo filename. 
    \return
    - Un nmero distinto de cero si filename no es un nombre vlido.
    - -EACCES Si el usuario que realiza la operacin no tiene permiso de lectura sobre el archivo filename.
    - Lo que devuelva cp_aclstat() en cualquier otro caso.
*/

asmlinkage long sys_aclstat(char * filename, struct acl_entry * acl_statbuf, int len){
	struct nameidata nd;
	int error;

	error = user_path_walk(filename, &nd);
	if(error)
	    goto out;


	error = permission(nd.dentry->d_inode, ACL_READ_BIT);
	if(error)
		goto path_relase_and_out;


	error = do_revalidate(nd.dentry);
	if (error)
	    goto path_relase_and_out;

	error = cp_aclstat(nd.dentry->d_inode, acl_statbuf, len);

path_relase_and_out: path_release(&nd);

out: return error;


}

/*===============================Lisex=======================================*/

/*!
	\ingroup newsyscall

	\brief Llamada al sistema aclfstat. Devuelve informacin de una ACL.
	
    Devuelve informacin de la ACL del inodo al que apunta el descripor de archivo que se pasa como parmetro.
    Esta llamada utiliza la funcin cp_aclstat().
    
    \param fd Descriptor de archivo que apunta al inodo del cual se piede la informacin.
    \param acl_statbuf Apunta a una estructura donde se devuelven los resultados de sys_fstat().
	\param len Cantidad de entradas de la ACL que se piden obtener como resultado.

    \post dem sys_aclstat()

    \return dem sys_aclstat()

*/
asmlinkage long sys_aclfstat(unsigned int fd, struct acl_entry * acl_statbuf, int len)
{
	struct file * f;
	int error = -EBADF;
	struct dentry * dentry;
	
	f = fget(fd);
	if (!f) 
	    goto out;
	
	dentry = f->f_dentry;

	error = permission(dentry->d_inode, ACL_READ_BIT);
	if(error)
		goto fputf_and_out;

	
	error = do_revalidate(dentry);
	
	if (error)
	    goto fputf_and_out;
	
	error = cp_aclstat(dentry->d_inode, acl_statbuf, len);	

fputf_and_out: 
	fput(f);	
out:	
	return error;

}

