/*===============================Lisex=======================================*/

/*! \file fs_acl.h
    \brief Definiciones, estructuras y prototipos de funciones que manejan las ACL de los inodos del VFS.
*/

#ifndef _LINUX_FS_ACL_H
#define _LINUX_FS_ACL_H
#include <linux/types.h>

/****************************************************************************/
/*********** Definiciones posiblemente chotas o mal ubicadas ****************/
/*Esto deberi'a sacarse del superbloque de turno y en realidad ser dinamico*/

/*!
	\addtogroup defines
*/
/*@{*/

//! Cantidad mxima de entradas posibles a la ACL de un inodo del VFS.
#define MAX_ACL_LEN 8

//! Identificacin del grupo AllGrp.
#define ALLGRP 666	/*mmmmmmmmmmmmmmm*/

//! Identificacin del usuario Root.
#define ROOTID	0

//! Identificacin del grupo Root.
#define ROOTGRP	0
/*@}*/


/****************************************************************************/
/********************  Definiciones generales *******************************/

//ACL_OK se documenta tambin en acl.c (mantener consistente)
/* 
	Retorno de funciones que manejan las ACL.
	
	Indica que la operacin se realiz con xito. 
*/
#define  ACL_OK           0

/****************************************************************************/
/****************   Para manejar el mode en las acl *************************/


/*!
	\addtogroup acl_entry

	\brief \a %acl_entry representa una terna de la forma (ID, MODO, USR_GRP). 
	
	Este TAD oculta la representacin de las ternas (ID, MODO, USR_GRP) que se usarn en las ACL a nivel del VFS, as como los algoritmos que se usan para obtener los atributos privados. 
	
	La estructura oculta \a acl_entry no debe ser accedida excepto por las funciones de la interfaz aqu documentada. Como esto no es posible imponerlo en tiempo de compilacin, los programadores son los responsables de mantener esta disciplina.
*/
/*@{*/

/*!
	\name Constantes (pblicas) de acl_entry
	
	Debe considerarse que el nombre de la constante es pblico y su valor privado.
*/
/*@{*/

//! Para representar el modo nulo (ningun permiso) en la ACL.
#define ACL_NULL_BITS 		0	

//! Para representar el permiso de ejecutabilidad en la ACL.
#define ACL_EXEC_BIT		1

//! Para representar el permiso de escritura en la ACL.
#define ACL_WRITE_BIT		2

//! Para representar el permiso de lectura  en la ACL.
#define ACL_READ_BIT 		4

//! Para representar el permiso de propiedad (dueo) en la ACL.
#define ACL_OWNER_BIT		8

//! Para representar el modo que tiene todos los permisos y tambin es dueo
#define ACL_RWXO_BIT		ACL_EXEC_BIT|ACL_READ_BIT|ACL_WRITE_BIT|ACL_OWNER_BIT


/****************************************************************************/
/********* Para manejar el campo type (usado/no_usado; grupo/usuario) *******/

/*! 
	\brief Para indicar que se trata de los permisos de un grupo en una entrada de la ACL.

    Se utiliza para establecer el campo acl_entry::type de la estructura ACL.
*/
#define ACL_GRP		0

/*! 
	\brief Para indicar que se trata de los permisos de un usuario en una entrada de la ACL.

    Se utiliza para establecer el campo acl_entry::type de la estructura ACL.
*/
#define ACL_USR		1
/*@}*/

/****************************************************************************/
/************************  struct acl_entry  ********************************/

/*! 
	\brief Entrada de la ACL a nivel VFS. Se utiliza dentro del la estructura vfs_acl.

	\remark Esta estructura debe ser accedida slo a travs de las funciones de su interfaz.
*/

struct acl_entry	/* Access Control List Entry */
{
	/*!Determina si la entrada corresponde a un usuario o grupo. Depender del campo \a type si \a id es 
	\a uid o \a gid.
	*/
	union {
#ifdef __KERNEL__
//#if 1
		/*!Identifica a un usuario.*/
	    uid_t uid; 
	    /*!Identifica a un grupo.*/
	    gid_t gid; 
#else
/*	    __uid_t uid;
	    __gid_t gid;*/
	    int uid;
	    int gid;
#endif
	} id;
		    
	/*!Se utiliza para representar los permisos del usuario/grupo (id) sobre el inodo.*/
	char	 	mode;	/* Access permissions */	
	
	/*!Puede tomar los valores #ACL_USR o #ACL_GRP, dependiendo si se trata de una entrada para un usuario
		o para un grupo.
	*/
	char 		type;	/* User or group entry*/
};


/****************************************************************************/
/*********************  Para manejar acl_entry  *****************************/
/*!
	\name Funciones de la interfaz de acl_entry
	
*/
/*@{*/

//! Devuelve el campo \a type de una entrada de ACL.
#define ACL_TYPE(e) e.type

//! Devuelve el campo \a mode de una entrada ACL.
#define ACL_MODE(e) e.mode

/*! 
	\brief Devuelve el el campo \a id de una entrada ACL.

	Como acl_entry::id es una union puede retornarse un entero que representa un UID o GID, dependiendo del valor de acl_entry::type.
*/
#define ACL_ID(e)  ((ACL_TYPE(e)==ACL_USR)?e.id.uid:e.id.gid)

/*@}*/
/*@}*/


#ifdef __KERNEL__
/****************************************************************************/
/************************  struct vfs_acl  **********************************/

/*! 
	\addtogroup vfs_acl
	
	\brief \a %vfs_acl representa la ACL a nivel del VFS. 
	
	Este TAD oculta la representacin de las listas de control de acceso a nivel del VFS, as como los algoritmos que se usan para establecer y obtener los atributos privados. 
	
	La estructura oculta \a vfs_acl no debe ser accedida excepto por las funciones de la interfaz aqu documentada. Como esto no es posible imponerlo en tiempo de compilacin, los programadores son los responsables de mantener esta disciplina.
*/
/*@{*/

/*!
	\brief ACL a nivel VFS. Se utiliza dentro del inodo del VFS.
	
	Esta estructura debe ser accedida slo a travs de las funciones de su interfaz.
*/
struct vfs_acl {
    struct acl_entry acl_entry[MAX_ACL_LEN]; /*!< Arreglo de #MAX_ACL_LEN estructuras acl_entry.*/
    int size; /*!<Representa la cantidad de entradas vlidas en el arreglo anterior.*/   
};
/*@}*/


/****************************************************************************/
/********************  Para manejar vfs_acl  ********************************/

// Inicializa una ACL.
void acl_aclinit(struct vfs_acl * vfs_acl);

// Aun no esta implementada, pero debera liberar la ACL que recibe como parmetro.
void acl_acldestroy(struct vfs_acl * vfs_acl);

// Establece el modo de un usuario o grupo en la ACL.
int acl_setmode(struct vfs_acl * vfs_acl, int id, int mode, int type);

// Devuelve la cantidad de entradas vlidas que existen en la ACL.    
int acl_getsize(struct vfs_acl * vfs_acl);

// Obtiene el modo de un usuario/grupo.
int acl_getmode(struct vfs_acl * vfs_acl, int id, int type);

// Calcula los permisos compatibles con el modo utilizado en una ACL a partir de un modo tradicional y de un entero que indica si son permisos de dueo o no. 
int xmode_to_acl(int xmode, int owner);

// Calcula el modo tradicional de un inodo a partir de su ACL.
umode_t acl_calc_xmode(struct vfs_acl * vfs_acl, uid_t uid, gid_t gid);


/****************************************************************************/
/*********** Definiciones para el iterador de acls entries ******************/

//! Renombre del tipo int, utilizado como iterador dentro de una ACL.
typedef int acl_iterator ;

//ACL_IT_ERROR y acl_for_each se documentan nuevamente en acl.c (mantener consistente)
// Retorno de las funciones que iteran sobre la ACL de un inodo.
/*
    Indica que no existen ms entradas vlidas en la ACL a partir de la posicin pasada como parmetro.
*/
#define ACL_IT_ERROR	-1

// Macro que itera entre todas las entradas vlidas de una ACL. 
/* 
    \param acl ACL sobre la cual se va a iterar.
    \param i Es el puntero iterador.
*/
#define acl_for_each(acl,i) \
  for(i=acl_first(acl);i!=ACL_IT_ERROR;i=acl_next(acl,i))

// Busca la primer entrada vlida dentro de la ACL.
acl_iterator acl_first(struct vfs_acl * vfs_acl);

// Busca la prxima entrada vlida en la ACL a partir de una posicin que se pasa como parmetro.
acl_iterator acl_next(struct vfs_acl * vfs_acl, acl_iterator it);

// Devuelve una entrada de la ACL.
struct acl_entry acl_getentry(struct vfs_acl * vfs_acl, acl_iterator it);

#endif
#endif
