/*===============================Lisex1=======================================*/
/*! \file
    \brief Definiciones, estructuras y prototipos de funciones para manejar las clases de seguridad (SC) en general.
*/


#ifndef _LINUX_SC_H
#define _LINUX_SC_H
#include<linux/errno.h>

//! Cantidad maxima de categorias posibles en la SC de un inodo del VFS.
#define MAX_CAT_LEN	 8

/*!
	\addtogroup sc
	
	\brief \a %sc representa las clases de seguridad a nivel de VFS. 
	
	Este TAD oculta la representacin de las clases de acceso (o de seguridad) a nivel de VFS, as como los algoritmos que se usan para establecer y obtener los atributos privados. 
	
	La estructura oculta \a sc no debe ser accedida excepto por las funciones de la interfaz aqu documentada. Como esto no es posible imponerlo en tiempo de compilacin, los programadores son los responsables de mantener esta disciplina. En caso de que no existan las funciones apropiadas para acceder los atributos privados, los programadores son libres de hacerlo como lo crean conveniente.
*/
/*@{*/

/*!
	\name Constantes (pblicas) de sc
	
	Debe considerarse que el nombre de la constante es pblico y su valor privado.
*/
/*@{*/

/*!
	\brief Retorno de funciones que manejan las SC.

    Indica que la operacin no pudo realizarse porque la SC del inodo est llena.
*/
#define SC_CAT_FULL	-1

/*! 

	\brief Retorno de funciones que manejan las SC.

    Indica que la operacin se realiz con xito.
*/
#define SC_OK		 0

//! Categora que deben tener los administradores de seguridad para cambiar clases de acceso de sujetos y objetos.
#define SCADMIN 65535
/*@}*/


/*!
	\brief SC a nivel VFS. Se utiliza dentro del inodo del VFS.
	
	Esta estructura debe ser accedida slo a travs de las funciones de su interfaz.
*/

struct sc{
    int level; /*!<Nivel de la SC.*/
    int categories[MAX_CAT_LEN]; /*!<Las categoras de la SC.*/
    int size;	/*!<Cantidad de categoras vlidas que estn almacenadas en \a categories.*/
};

typedef struct sc sc;

// Libera los recursos asignados a una SC.
void sc_destroy(sc * sc);

// Inicializa una SC.
sc * sc_init(void);

// Obtiene la cantidad de categoras vlidas dentro de una SC.
int sc_getsize(sc * sc);

// Obtiene el nivel de una SC.
int sc_getlevel(sc * sc);

// Establece el nivel de una SC.
void sc_setlevel(sc * sc, unsigned int l);

// Borra todas las categoras de una SC.
void sc_clearcat(sc * sc);

// Agrega una categora las categoras de una SC.
int sc_addcat(sc * sc, int cat);

// Determina si la SC \a sc2 domina la SC \a sc1.
int sc_compare(sc * sc1, sc * sc2);

// Copia el contenido de  una clase de seguridad en otra.
int sc_copy(sc * sc1, sc * sc2);

//  No comentado!!
void sc_print(struct sc * sc);

#ifdef __KERNEL__
// Carga un conjunto de categoras que provienen del espacio de usuario a una SC.
int sc_usergetncat(sc * sc, int *cat, int n);

// Carga un conjunto de categoras que provienen de una SC en espacio de kernel a un conjunto de categoras que se encuentra en espacio de usuario.
int sc_userputcat(sc * sc, int * cat);
#else
#include <stdio.h>
#define printk printf
#define panic(x) 
#define LISEXDEBUG(x)
#define LISEXDEBUG2(x)
#endif


//! Renombre del tipo int, utilizado como iterador dentro de las categoras de una SC.
typedef int sc_cat_iterator ;


//SC_IT_ERROR y sc_for_each_cat se documentan tambin en sc.c (mantener consistentes)
// Retorno de las funciones que iteran sobre las categoras de una SC de un inodo.
//Indica que no existen ms entradas vlidas en la ACL a partir de la posicin pasada como parmetro.
#define SC_IT_ERROR	-1

// Macro que itera entre todas las categoras de una SC. 
/*
    \param sc SC que tiene las categorias sobre las que se va a iterar
    \param i Es el puntero iterador.
    \bug el 'arreglo' de categorias podrian ser una ED por si misma
*/
#define sc_for_each_cat(sc,i) \
  for(i=sc_cat_first(sc);i!=SC_IT_ERROR;i=sc_cat_next(sc,i))

// Obtiene un iterador o selector que corresponde a la primer categora vlida dentro de las categoras de una SC.
sc_cat_iterator sc_cat_first(sc * sc);

// Obtiene el iterador correspondiente a la prxima categora vlida dentro de una SC a una posicin de referencia pasada como parmetro.
sc_cat_iterator sc_cat_next(sc * sc, sc_cat_iterator it);

// Obtiene la categora referenciada por un iterador.
int sc_getcat(sc * sc, sc_cat_iterator it);

/*@}*/

#endif  /*_LINUX_SC_H*/
