/*===============================Lisex1=======================================*/
/*! 
	\file 
	\brief Definiciones de las funciones declaradas en sc.h y algunas funciones auxiliares.

 	\bug Los conjuntos de categoras deben implementarse con arreglos o conjuntos de bits
      cada uno representando una categora, ver capability.h.
*/

/*
#include<linux/init.h>
#include<linux/kernel.h>
#include<asm/atomic.h>
#include <linux/module.h>
#include <linux/spinlock.h>
*/

#include<linux/sc.h>

#ifdef __KERNEL__
#include <linux/slab.h>
#else
#include <stdlib.h>
#define kmalloc(x,y) malloc(x)
#define kfree   free
#define BUG() 
#endif

/*! \ingroup structdef */
/*!
	\addtogroup sc
*/
/*@{*/
/*!
	\name Iterador para las categoras de un objeto sc
	
	Tambin forma parte de la interfaz de sc.
	
	Ver #sc_for_each_cat para un ejemplo que muestra cmo usarlo.
*/
/*@{*/

//SC_IT_ERROR y sc_for_each se documentan tambin en sc.h (mantener consistentes)
/*!
	\def SC_IT_ERROR
	
	\brief Retorno de las funciones que iteran sobre las categoras de una SC de un inodo.
	
	Indica que no existen ms entradas vlidas en la ACL a partir de la posicin pasada como parmetro.
*/

/*!
	\def sc_for_each_cat(sc,i)
	
	\brief Macro que itera entre todas las categoras de una SC. 

    \param sc SC que tiene las categoras sobre las que se va a iterar
    \param i Es el puntero iterador.

    \bug el 'arreglo' de categoras podra ser un TAD por s mismo.
*/

/*!
	\example iterador_sc_cat.c
	
	Este ejemplo muestra cmo usar el iterador para recorrer las categoras en una clase de acceso.
*/

/*! 
	\brief Obtiene un iterador o selector que corresponde a la primer categora vlida dentro de las categoras de una SC.
	
	\param sc SC en la cual se busca la primer categora vlida.

    \return 
	    - 0 Si la cantidad de categras vlidas en \a %sc es mayor que cero.
	    - #SC_IT_ERROR Si no existe ninguna categora vlida en \a %sc.
*/
sc_cat_iterator sc_cat_first(struct sc * sc){
    if(sc->size>0)
		return 0;
    return SC_IT_ERROR;
}


/*! 
	\brief Obtiene el iterador correspondiente a la prxima categora vlida dentro de una SC con respecto a una posicin de referencia pasada como parmetro.

    \param sc SC en la cual se busca la prxima categora vlida.
    \param it Posicin dentro de las categoras de \a %sc donde se encuentra la categora de referencia.

    \return
    	- \a it+1 Si hay suficientes categoras vlidas en \a sc.
        - #SC_IT_ERROR Si no existe ninguna categora vlida luego de la referenciada por \a it.
*/
sc_cat_iterator sc_cat_next(struct sc * sc, sc_cat_iterator it){
    if(it+1 < sc->size)
	return it+1;
    return SC_IT_ERROR;
}


/*! 
	\brief Obtiene la categora referenciada por un iterador.

    La SC debe ser vlida y el iterador debe ser uno obtenido utilizando
    sc_cat_first() y sc_cat_next() sobre la misma SC y distinto de #SC_IT_ERROR.
    
    \param sc SC donde se encuentra la categora a devolver.
    \param it Iterador correspondiente a la categora que se desea obtener.

    \return La categora ubicada en la posicin \a it de \a %sc.
*/
int sc_getcat (struct sc *sc, sc_cat_iterator it)
{
  if (it == SC_IT_ERROR)
    BUG();
  return sc->categories[it];
}
/*@}*/


/*!
	\name Funciones de la interfaz de sc	
*/
/*@{*/

/*! 
	\brief Libera los recursos asignados a una SC.

	Destruye memoria del ncleo utilizada en una SC.
    
    \param  sc SC que ser eliminada.
*/
void sc_destroy (struct sc *sc)
{
    kfree(sc);
};


/*! 
	\brief Inicializa una SC.

	Se debe tener en cuenta lo siguiente:
	- Esta funcin prepara y reserva recursos para que un objeto sc pueda ser utilizado.
	- Ningn objeto sc debe ser utilizado sin haber ejecutado antes esta funcin.
	- Inicializa a la clase de seguridad ms baja.
*/
struct sc * sc_init(void)
{
  struct sc * sc;
  sc = (struct sc *)kmalloc(sizeof(struct sc), GFP_KERNEL);
  sc->size = 0;
  sc->level = 0;
  return sc;
};


/*! 
	\brief Obtiene la cantidad de categoras vlidas dentro de una SC.

    \param sc SC sobre la cual se realiza la operacin.

    \return 
	- Cantidad de categoras vlidas, siempre que exista alguna.
	- -1 Si no existen categoras vlidas.
*/
int sc_getsize (struct sc *sc)
{
  return sc->size;
};


/*! 
	\brief Obtiene el nivel de una SC.
	
	El parmetro \a %sc debe estar inicializado.

    \param sc SC de la cual se obtiene el nivel.

    \return El nivel de \a %sc.
*/
int sc_getlevel(struct sc * sc)
{
    return sc->level;
};


/*! 
	\brief Establece el nivel de una SC.
	
	El parametro \a %sc debe estar inicializado.

    \param sc SC a la cual se asignar el nivel.
    \param l El nuevo nivel.
*/
void sc_setlevel(struct sc * sc, unsigned int l)
{
    sc->level=l;
};


/*! 
	\brief Borra todas las categoras de una SC.

    \param sc SC de la cual se borrarn todas las categoras.
*/
void sc_clearcat (struct sc *sc)
{
  sc->size = 0;
}


/*! 
	\brief Agrega una categora las categoras de una SC.

    \param sc SC sobre la cual se realizar la insercin de una categora.
    \param cat Categora a agregar a las categoras de \a %sc.

    \return 
	    - #SC_CAT_FULL Si no hay ms lugar para agregar una categora a las categorias de \a %sc.
	    - #SC_OK Si la operacin se realiz con xito.

    \bug Siempre agrega la categora por ms que ya est.
*/
int sc_addcat(struct sc * sc, int cat){
        if(!cat_in(cat,sc)){
	    if(sc->size+1 > MAX_CAT_LEN)
		return SC_CAT_FULL;
	    else
		sc->categories[sc->size++]=cat;
	}
        return SC_OK;
}


/*! 
	\brief Determina si \a sc2 domina a \a sc1.

    \param sc1 Primer parmetro de la comparacin.
    \param sc2 Segundo parmetro de la comparacin.

    \return
	    - 1 Si \a sc2 no domina a \a sc1.
	    - 0 Si \a sc2 domina a \a sc1.
    
    \bug Si alguna de las SC a comparar no son vlidas, el resultado es impredecible.
*/
int sc_compare (struct sc *sc1, struct sc *sc2)
{
    sc_cat_iterator j;
    if (!sc1 || !sc2)
	panic ("sc_compare: Bad arguments.\n");
    if (sc_getlevel(sc1) > sc_getlevel(sc2)) 
	return 1;
    sc_for_each_cat(sc1,j)
        if(!cat_in(sc_getcat(sc1,j),sc2))
	    return 1;
    return 0;	
};


/*! 
	\brief Copia el contenido de  una clase de seguridad en otra.

    \param sc1 SC destino.
    \param sc2 SC origen.

    \return
	    - #SC_OK Si la operacin se realiz con xito.
	    - #SC_CAT_FULL Si ocurri algn error.
*/
int sc_copy(struct sc * sc1, struct sc * sc2)
{
	sc_cat_iterator i;
	int error = SC_OK;
	sc_clearcat(sc1);
//	for (i = sc_cat_first(sc2); i != SC_IT_ERROR ;i = sc_cat_next(sc2, i))
	sc_for_each_cat(sc2,i)
		if((error = sc_addcat(sc1, sc_getcat(sc2 , i))))
			break;
	sc_setlevel(sc1, sc_getlevel(sc2));
return error;
}

#ifdef __KERNEL__
#include<asm/uaccess.h>
/*! 
	\brief Carga un conjunto de categoras que provienen del espacio de usuario a una SC.

    \param sc SC a la cual se le cargarn las categoras.
    \param cat Conjunto de categoras a cargar en \a %sc.
    \param n Cantidad de categoras vlidas que estan en \a cat.

    \return 
	    - #SC_CAT_FULL Si no es posible cargar \a n categoras a \a %sc porque no hay al menos \a n lugares libres.
	    - 0 Si la operacin se realiz con xito
	    - Un nmero distinto de 0 si no se pudo concretar la operacin por algn otro motivo.    
*/
int sc_usergetncat(struct sc * sc, int *cat, int n)
{
  int error;
  if (sc->size + n > MAX_CAT_LEN)
      return -SC_CAT_FULL;
    error = copy_from_user (sc->categories + sc->size, cat, n * sizeof (int));
  if (!error)
      sc->size += n;
    return error;
}


/*! 
	\brief Carga un conjunto de categoras que provienen de una SC en espacio de kernel a un conjunto de categoras que se encuentra en espacio de usuario.

    No se realiza ningn control para asegurar que la cantidad de categoras en 
    la SC de espacio de kernel es menor o igual a la cantidad de espacio reservado
    en espacio de usuario para almacenarlas.
    
    \param sc SC de la cual se le obtendrn las categoras.
    \param cat Conjunto de categoras a cargar en \a %sc.

    \return 
	    - 0 Si la operacin se realiz con xito.
	    - Un nmero distinto de 0 si no se pudo concretar la operacin por algn motivo.
*/
int sc_userputcat(struct sc * sc, int *cat)
{
  return copy_to_user (cat, sc->categories, sc->size * sizeof (int));
};
/*@}*/
/*@}*/

/*!
	\name Funciones adicionales
	
	Funciones implementadas con las funciones de la interfaz.
*/
/*@{*/


#include<linux/subjectsc.h>
/*! 
	\brief Determina si un usuario es un administrador de seguridad

    Determina si el usuario que llama a esta funcin es o no un oficial MAC.

    \return
	    - 1 Si es oficial MAC.
	    - 0 Si no es oficial MAC.
	
	\bug No debera ir en este archivo!
*/
int is_secadm(void)
{
    return cat_in(SCADMIN, subjectscget(current->fsuid));
}

#endif


/*! 
	\brief Operador de pertenencia sobre conjuntos de categoras.
	
	Implementa la operacion de pertenencia de una categoria al conjunto de 
    categorias de una SC.
    
    Determina si \a cat pertenece a \a sc o no.
    
    \param cat Una categora.
    \param sc Una clase de seguridad.

    \return 
	    - 1 Si \a cat pertenece a \a %sc. 
	    - 0 Si \a cat no pertenece a \a %sc .
*/
int cat_in (int cat, sc * sc){
sc_cat_iterator i;
	
    sc_for_each_cat(sc,i)
	        if(sc_getcat(sc,i)==cat)
			return 1;	
	return 0;    
}

/*@}*/

/*
    \bug No comentado
*/
void sc_print(struct sc * sc)
{
    sc_cat_iterator i;
    printk("Level: %d\n",sc->level);
    printk("Cats:");
    sc_for_each_cat(sc,i)
	printk("%d ", sc_getcat(sc , i));
    printk("\n");
}

