/*===============================Lisex1=======================================*/
/*! \file 
 \brief Definiciones de las funciones declaradas en subjectsc.h y algunas funciones auxiliares.
*/

#ifndef __KERNEL__
#define __init
#define __exit 
#define printk printf 
#else
#include<linux/init.h>
#include<linux/kernel.h>
#endif

#include<linux/subjectsc.h>

/*!
	\addtogroup defines
*/
/*@{*/

/*! 
	\brief Maneja los mensajes del ncleo.
	
	Se utiliza durante la fase de correccin para manejar los de mensajes de las porciones del ncleo que fueron modificadas o agregadas por Lisex.
*/
#define LISEXDEBUG(x) 

/*! 
	\brief Cantidad mxima de SC de sujetos. 

	Se considera que los usuarios son aquellos que tienen UID menor que \a SC_MAX_BUCKETS. Esto se debe a que Lisex 0.0 es un prototipo.
*/
#define SC_MAX_BUCKETS 600

//! Funcin de dispersin; por ahora un arreglo completo.
#define SUBSC_HASH(x) x
/*@}*/


/*! 
	\addtogroup subjectsc
*/
/*@{*/

/*!
	\name Estructuras de datos privadas (ocultas)
*/
/*@{*/

/*! 
	\brief Aqu se almacenan las SC de los sujetos.
	
	Esta estructura debe ser accedida slo a travs de las funciones de su interfaz.
	
	Ver #SC_MAX_BUCKETS.
*/
static sc * subjectsc[SC_MAX_BUCKETS];

/*! 
	\brief Apunta a la SC ms baja.
	
	Usado por todos los sujetos que no tengan SC asignada. Una vez que se inicializa #subjectsc (subjectsc_init()) todas sus componentes apuntan a \a lowest_sc, de forma tal que cada nuevo usuario tendr \a lowest_sc como clase de acceso hasta tanto se le asigne una.
*/
static sc * lowest_sc;
/*@}*/


/*---------------para inicializar y destruir los datos etaticos del modulo*/

/*!
	\name Funciones de la interfaz de subjectsc
*/
/*@{*/

/*!
	\brief Inicializa las SC de todos los sujetos.
	
    Se considera que los usuarios son aquellos que tienen UID menor que #SC_MAX_BUCKETS. Esto se debe a que Lisex 0.0 es un prototipo; en futuras versiones se inicializar la clase de acceso de cada usuario presente en el sistema (i.e. en \c /etc/passwd).
*/
//static
int __init subjectsc_init (void)
{
  int i;
  lowest_sc = sc_init();
  for (i = 0; i < SC_MAX_BUCKETS; i++)
	subjectsc[i] = NULL;
  
  LISEXDEBUG(printk("SUBJECTSC: Initializing....\n"));
  
  return 0;
}


//! Libera las SC de todos los sujetos.

//static
int __exit subjectsc_exit (void)
{
  int i;
  for (i = 0; i < SC_MAX_BUCKETS; i++)
    if(subjectsc[i])
	/*borrar toda la lista*/ 
	sc_destroy (subjectsc[i]);

  LISEXDEBUG(printk("SUBJECTSC: Destroying....\n"));
  return 0;
}


/*-----------Para manejar lo que en un futuro sera una hash ----------*/

/*!    
	\brief Obtiene la SC de un usuario.
	
	Se considera que los usuarios son aquellos que tienen UID menor que #SC_MAX_BUCKETS. Esto se debe a que Lisex 0.0 es un prototipo; en futuras versiones se inicializar la clase de acceso de cada usuario presente en el sistema (i.e. en \c /etc/passwd).
	
    \param uid Identificacin del usuario sobre el que se quiere realizar la operacin.

    \return La SC del usuario con identificacin \a uid.

    \bug Debera buscar dentro del bucket (una lista enlazada?)
*/
sc * subjectscget (uid_t uid)
{
//  return /*find in*/subjectsc[SUBSC_HASH(uid)];
    return subjectsc[SUBSC_HASH(uid)]?subjectsc[SUBSC_HASH(uid)]:lowest_sc;
}


/*! 
	\brief Agrega una categora a la SC de un usuario.
	
    \return 
    	- -EPERM si \a uid no es un usuario registrado (ver subjectsc_init()).
    	- 0 si la operacin se realiz con xito.
*/
int subjectscadd (uid_t uid, sc *sc)
{
    if(subjectsc[SUBSC_HASH(uid)]!=NULL){
	    /*buscar en la lista plin plin plin ...*/
	    return -EPERM;
    }
    /*Inseratar en la lista */
    subjectsc[SUBSC_HASH(uid)]=sc; /*en realidad hay que crear un nodo con link o agregar a la estructura sc los links para poder usarla como lista del kernel?*/
    return 0;
}


//! Elimina una categora de la SC de un usuario.

sc * subjectscdel (uid_t uid)
{
    sc * aux = subjectsc[SUBSC_HASH(uid)];
    subjectsc[SUBSC_HASH(uid)]=NULL;	
    return aux;
}
/*@}*/
/*@}*/


#ifdef __KERNEL__
#include<linux/init.h>
#include<asm/uaccess.h>

//Dice al kernel que llame a subjectsc_init al inicio y a subjectsc_exit al salir
module_init (subjectsc_init);
module_exit (subjectsc_exit);



/*!
	\ingroup newsyscall
	
	\brief Llamada al sistema chsubsc. Cambia la clase de acceso de un usuario.
	
	\pre 
		- El usuario que invoca la llamada debe ser un oficial MAC.
		- El usuario al cual se le cambiar la clase de acceso no debe tener archivos abiertos.
	
    \param uid Identificacin del usuario sobre el cual se realiza la operacin.
    \param level Nivel que se le asignar a la SC del usuario.
    \param categories Conjunto de categoras que se le asignarn a la SC del usuario.
    \param size Cantidad de categoras que se le asignarn a la SC del usuario.
    
    \post
    	- SC de \a uid cambia por una nueva SC con nivel \a level y conjunto de categoras \a categories.
    
    \return
    	- -EBUSY Si \a uid esta corriendo algn proceso en el momento de realizar la operacin.
     	- -EACCES Si el usuario que realiza esta operacin no pertenece al grupo SecAdm.
    	- 0 Si la operacin se realiz con xito.
*/

asmlinkage long
sys_chsubsc (uid_t uid, int level, int *categories, int size)
{
  int error;
  
  if (logged (uid))
    return -EBUSY;
    if(!is_secadm())
	    return -EACCES;
  sc_clearcat (subjectscget(uid));
  sc_setlevel (subjectscget(uid), level);
  error = sc_usergetncat (subjectscget(uid), categories, size);
  return error;
}


/*!
	\ingroup newsyscall
	
	\brief Llamada al sistema sscstat. Retorna la clase de acceso de \a uid.
	
    \param uid Identificacin del usuario del cual se quiere conocer la clase de seguridad.
    \param level Donde se cargar el nivel de la clase de acceso de \a uid.
    \param categories Donde se cargarn las categoras de la clase de acceso de \a uid.
    \param size Cantidad de categoras que se pide cargar en categories.
    
    \post
	    - El parmetro \a level contiene el nivel de la clase de acceso de \a uid.
		- En el parmetro \a categories estn las categoras de la clase de acceso de \a uid, si \a size es mayor que 0.
		 
    \result
		- -EACCES Si La clase de acceso del usuario que realiza la operacin no domina a la de \a uid.
		- Cantidad de entradas vlidas que hay en la clase de acceso de \a uid, si \a size es 0. En este caso se carga slo el nivel. 
		- -EINVAL Si \a size es menor que la cantidad de entradas vlidas que hay en la clase de acceso de \a uid.
		-  m Si la operacin se realiz con xito. \a m es la cantidad de entradas vlidas que se cargaron en \a categories. 
*/

asmlinkage long
sys_sscstat (uid_t uid, int *level, int *categories, int size)
{
  int error;
  int tmp_lev;
  tmp_lev = sc_getlevel (subjectscget(uid));

  error = -EACCES;
  if (sc_compare (subjectscget(uid), subjectscget (current->fsuid)) )
    goto out;


  error = sc_getsize (subjectscget(uid));
  if (size == 0)
    {
      copy_to_user (level, &tmp_lev, sizeof (int));
      goto out;
    }

  error = -EINVAL;
  if (size < sc_getsize (subjectscget(uid)))
      goto out;


  error = copy_to_user (level, &tmp_lev, sizeof (int));
  error |= sc_userputcat (subjectscget(uid), categories);

  error = sc_getsize (subjectscget(uid));
out:
  return error;
}


/*! 
	\brief Determina si un sujeto est corriendo algn proceso en el sistema.
	
	\ingroup newauxfunc

    Determina si un usario est logueado (corriendo algn proceso) o no en el sistema.

    \param uid Usuario sobre el cual se realiza la operacin.
    
    \return 
    - 0 Si el usuario con identificacin \a uid no est corriendo ningn proceso en el sistema.
    - 1 Si el usuario est corriendo al menos un proceso en el sistema.
*/

int logged (uid_t uid)
{
    int ret;
    struct user_struct *u;
    u = alloc_uid(uid);
    if(!u)
	return 0;
    ret = atomic_read(&u->processes);
    free_uid(u);
    return ret>0;
}
#endif
